@@ -51,16 +51,14 @@ def test_automatic_creation(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -112,16 +110,14 @@ def test_enter_project_directory(hatch, config_file, helpers, temp_dir):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -166,16 +162,14 @@ def test_sync_dependencies(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -241,16 +235,14 @@ def test_sync_project_dependencies(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -322,16 +314,14 @@ def test_sync_project_features(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -404,16 +394,14 @@ def test_scripts(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -469,16 +457,14 @@ def test_scripts_specific_environment(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -627,16 +613,14 @@ def test_ignore_error(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -740,16 +724,14 @@ def test_verbosity(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -835,16 +817,14 @@ def test_matrix(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = sorted(storage_path.iterdir(), key=lambda d: d.name)[::-1]
     assert len(env_dirs) == 2
@@ -983,16 +963,14 @@ def test_incompatible_matrix_partial(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -1050,16 +1028,14 @@ def test_incompatible_missing_python(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -1115,16 +1091,14 @@ def test_env_detection(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = sorted(storage_path.iterdir(), key=lambda d: d.name)
     assert len(env_dirs) == 2
@@ -1188,16 +1162,14 @@ def test_env_detection_override(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = sorted(storage_path.iterdir(), key=lambda d: d.name)
     assert len(env_dirs) == 2
@@ -1438,16 +1410,14 @@ def test_matrix_variable_selection_inclusion(hatch, helpers, temp_dir, config_fi
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -1501,16 +1471,14 @@ def test_matrix_variable_selection_exclusion(hatch, helpers, temp_dir, config_fi
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -1594,6 +1562,69 @@ def test_matrix_variable_selection_include_none(hatch, helpers, temp_dir, config
     )
 
 
+def test_matrix_variable_selection_inclusion_multiple_variables(hatch, helpers, temp_dir, config_file):
+    config_file.model.template.plugins['default']['tests'] = False
+    config_file.save()
+
+    project_name = 'My.App'
+
+    with temp_dir.as_cwd():
+        result = hatch('new', project_name)
+
+    assert result.exit_code == 0, result.output
+
+    project_path = temp_dir / 'my-app'
+    data_path = temp_dir / 'data'
+    data_path.mkdir()
+
+    project = Project(project_path)
+    helpers.update_project_environment(project, 'default', {'skip-install': True, **project.config.envs['default']})
+    helpers.update_project_environment(
+        project, 'test', {'matrix': [{'version1': ['9000', '42'], 'version2': ['3.14']}]}
+    )
+
+    with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
+        result = hatch(
+            'run',
+            '+version1=9000',
+            'test:python',
+            '-c',
+            "import os,sys;open('test.txt', 'a').write(sys.executable+os.linesep[-1])",
+        )
+
+    assert result.exit_code == 0, result.output
+    assert result.output == helpers.dedent(
+        """
+        ──────────────────────────────── test.9000-3.14 ────────────────────────────────
+        Creating environment: test.9000-3.14
+        Checking dependencies
+        """
+    )
+    output_file = project_path / 'test.txt'
+    assert output_file.is_file()
+
+    env_data_path = data_path / 'env' / 'virtual'
+    assert env_data_path.is_dir()
+
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
+    assert len(storage_dirs) == 1
+
+    storage_path = storage_dirs[0]
+    assert len(storage_path.name) == 8
+
+    env_dirs = list(storage_path.iterdir())
+    assert len(env_dirs) == 1
+
+    env_path = env_dirs[0]
+    assert env_path.name == 'test.9000-3.14'
+
+    python_path = str(output_file.read_text()).strip()
+    assert str(env_path) in python_path
+
+
 def test_context_formatting_recursion(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
@@ -1641,16 +1672,14 @@ def test_context_formatting_recursion(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
