@@ -1,7 +1,17 @@
+import pytest
+
 from hatch.project.core import Project
 from hatchling.utils.constants import DEFAULT_BUILD_SCRIPT
 
 
+@pytest.fixture
+def local_builder(mock_backend_process, mocker):
+    if mock_backend_process:
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.build_environment')
+
+    yield
+
+
 def test_incompatible_environment(hatch, temp_dir, helpers):
     project_name = 'My.App'
 
@@ -46,6 +56,32 @@ def test_show_dynamic(hatch, temp_dir):
     assert result.output == '0.0.1\n'
 
 
+@pytest.mark.usefixtures('local_builder')
+def test_show_dynamic_missing_build_dependencies(hatch, helpers, temp_dir):
+    project_name = 'My.App'
+
+    with temp_dir.as_cwd():
+        hatch('new', project_name)
+
+    path = temp_dir / 'my-app'
+
+    project = Project(path)
+    config = dict(project.raw_config)
+    config['build-system']['requires'].append('foo')
+    project.save_config(config)
+
+    with path.as_cwd():
+        result = hatch('version')
+
+    assert result.exit_code == 0, result.output
+    assert result.output == helpers.dedent(
+        """
+        Setting up build environment for missing dependencies
+        0.0.1
+        """
+    )
+
+
 def test_set_dynamic(hatch, helpers, temp_dir):
     project_name = 'My.App'
 
