@@ -45,8 +45,9 @@ class CustomHook(BuildHookInterface):
             Path(relative_root, 'PKG-INFO'),
             f"""\
 Metadata-Version: {DEFAULT_METADATA_VERSION}
-Name: {kwargs["project_name_normalized"]}
+Name: {kwargs['project_name']}
 Version: 0.0.1
+License-File: LICENSE.txt
 Requires-Dist: binary
 """,
         )
